#! /bin/sh
### ====================================================================
### This script is an interface to a large mgquery database of BibTeX
### bibliography files in the BibNet Project and TeX User Group (TUG)
### databases, the master archives for which reside at
###
### 	ftp://ftp.math.utah.edu/pub/bibnet
### 	ftp://ftp.math.utah.edu/pub/tex/bib/
###
### From version 2.00, it also provides access to other databases of
### BibTeX data, which, for reasons of quality, or subject area, are
### best kept separate from the combined BibNet Project and TUG data.
###
### Usage:
###	[env [ DEBUG=xxx ] \
###	     [ BIBSEARCHPATH=[:]dir[:dir:dir:...[:]] ]] \
###	bibsearch [ --bibsearchpath dir[:dir:dir:...] ] \
###		  [ --debug ] \
###		  [ --help ] \
###		  [ --quiet ] \
###		  [ --version ] \
###		  [ databasename ]
###
### [13-Feb-2001] -- version 2.00
### [18-May-1997] -- version 1.02
### [13-May-1997] -- version 1.01
### [12-Feb-1997] -- original version (1.00)
### ====================================================================

VERSIONNUMBER=2.00
VERSIONDATE='13-Feb-2001'

### ====================================================================
### Define the installation default database search path, and record the
### user's personal setting, if any.
DEFAULTBIBSEARCHPATH=/usr/local/src/bibdata
USERBIBSEARCHPATH=$BIBSEARCHPATH


### ====================================================================
### Prevent security attacks from redefinition of IFS!  The definition
### here is a 3-character string: SPACE TAB NEWLINE
IFS=" "
IFS="	$IFS"
IFS="
$IFS"
###??? echo DEBUG: "$IFS" | od -xba ; exit 0

PATH=/bin:/usr/bin:/usr/ucb:$PATH
export PATH

### ====================================================================
### For security reasons, all program names MUST be defined as
### absolute paths, or be found in system directories forced to
### occur first in the PATH

AWK=/usr/local/bin/mawk
CAT=/usr/local/bin/cat
FIND=/usr/local/bin/find
MGQUERY=/usr/local/bin/mgquery
SED=/usr/local/bin/sed
SORT=/usr/local/bin/sort
TEST=/bin/test
TRUE=/bin/true

### ====================================================================
### Make sure that MGQUERY exists: if not, tell the user where else to
### find it.

if $TEST ! -f $MGQUERY
then
	## rsh access works poorly if a pager is selected, so instead, just
	## tell the user to switch machines
	##	ssh xserver.math.utah.edu "env MGDATA=$BIBSEARCHPATH $MGQUERY $database"
	echo "The mgquery search engine is not installed on this machine."
	echo "Please switch to a local Sun system, such as xserver.math.utah.edu"
	exit 1
fi

### ====================================================================
### Shell functions: modern sh, plus bash and ksh support these.  If
### your sh is ancient, change the first line of this script to run a
### modern version of a sh-compatible shell.

debugreport()
{
	## Usage: debugreport message text

	$TEST -n "$DEBUG" && echo DEBUG: "$@" 1>&2
}

dircheck()
{
	## Usage: dircheck directoryname

	debugreport dircheck: $1

	## Do some sanity checks: even the standard BIBSEARCHPATH directories might
	## not exist on some systems!
	if $TEST ! -d $1
	then
		error "not a directory: $1"
	elif $TEST ! -x $1
	then
		error "not an executable directory: $1"
	elif $TEST ! -r $1
	then
		error "not a readable directory: $1"
	fi
}


dirpathcheck()
{
	## Usage: dirpathcheck dir[:dir:dir:...]

	debugreport dirpathcheck: $1

	for d in `echo $1 | $SED -e 's/:/ /g'`
	do
		dircheck $d
	done
}


expandpath()
{
	## Usage: newpath=`expandpath pathlist`
	## Replace an empty path element by $DEFAULTBIBSEARCHPATH

	newpath=`
		$AWK 'BEGIN {
				gsub("::",(":" ARGV[2] ":"), ARGV[1]);
				sub("^:",(ARGV[2] ":"), ARGV[1]);
				sub(":$",(":" ARGV[2]), ARGV[1]);
				print ARGV[1];
				exit(0);
			}' "$1" "$DEFAULTBIBSEARCHPATH"
	`
	debugreport expandpath: "$1" '->' "$newpath"
	echo $newpath
}


error()
{
	## Usage: error message-text

	usage
	echo FATAL ERROR: "$@"
	exit 1
}

findinpath()
{
	## Usage: findinpath directorypath dir-or-file-or-symlink
	##
	## Find dir-or-file-or-symlink in directorypath, and return
	## its full path name.  If dir-or-file-or-symlink is not
	## found, exit with a fatal error.

	debugreport findinpath: $1 $2

	for d in `echo $1 | $SED -e 's/:/ /g'`
	do
		f=$d/$2
		debugreport findinpath: checking $f
		if $TEST -d $f -o -f $f -o -s $f
		then
			debugreport findinpath: found $f
			echo $f
			return
		fi
	done
	error cannot find $2 in path $1
}

help()
{
	## Usage: help

	## Build a list of subdirectories of $BIBSEARCHPATH that
	## themselves contain a readable and executable mgdata
	## subdirectory, and show their names, and the contents of any
	## DESCRIPTION files that they contain.

	## Since we have to actually scan for files in the search
	## path, make sure that it is properly expanded first:
	setbibsearchpath

	dirpathcheck $BIBSEARCHPATH
	echo "System-defined BIBSEARCHPATH = $DEFAULTBIBSEARCHPATH"
	$TEST -n "$USERBIBSEARCHPATH" && \
	echo "User-defined   BIBSEARCHPATH = $USERBIBSEARCHPATH"
	echo "Final expanded BIBSEARCHPATH = $BIBSEARCHPATH"

	startdir=`pwd`
	bibdirs=
	for e in `echo $BIBSEARCHPATH | $SED -e 's/:/ /g'`
	do
		## We need to start at startdir, in case e is a relative
		## directory name:
		cd $startdir

		debugreport searching for top-level subdirectories of: $e

		subdirs=`$FIND $e/* -prune -type d 2>/dev/null | $SORT `

		if $TEST -n "$subdirs"
		then
			for d in $subdirs
			do
				debugreport checking access for: $d/mgdata
				if $TEST -d $d/mgdata -a -r $d/mgdata -a -x $d/mgdata
				then
					debugreport access okay to: $d/mgdata
					bibdirs="$bibdirs $d"
				fi
			done
		fi
	done
	cd $startdir
	if $TEST -z "$bibdirs"
	then
		error "no mgdata subdirectories found in BIBSEARCHPATH = $BIBSEARCHPATH"
	else
		usage
		debugreport bibdirs: $bibdirs
		echo 'The following bibliographic databases are available:'
		for d in $bibdirs
		do
			printf "\n\t%-15s\t%s\n" `basename $d` "`$CAT $d/DESCRIPTION 2>/dev/null || $TRUE`"
		done
	fi
}

setbibsearchpath()
{
	if $TEST -z "$BIBSEARCHPATH"
	then				# BIBSEARCHPATH not in environment or set from command line
		BIBSEARCHPATH=$DEFAULTBIBSEARCHPATH
	else
		BIBSEARCHPATH="`expandpath $BIBSEARCHPATH`"
	fi
}

usage()
{
	## Usage: usage

	echo ''
	echo 'Usage: bibsearch [ --bibsearchpath dir[:dir:dir:...] ] [ --debug ] [ --help ] '
	echo '                 [ --quiet ] [ --version ] [ databasename ]'
	echo ''
	echo 'Optional environment variables:  DEBUG BIBSEARCHPATH'
	echo ''
}

welcome()
{
$CAT <<END-OF-COMMENTS
========================================================================
Welcome to the BibTeX bibliography database search facility
Version $VERSIONNUMBER $VERSIONDATE

END-OF-COMMENTS

echo "Bibliography database: " $database
echo ""

if $TEST -f $MGDATA/STATS.date
then
	$AWK '{printf("Last database update:  %s\n",   $0)}' $MGDATA/STATS.date
	$AWK '{printf("Bibliography entries:  %10d\n", $1)}' $MGDATA/STATS.entries
	$AWK '{printf("Bibliography lines:    %10d\n", $1)}' $MGDATA/STATS.wc
	$AWK '{printf("Bibliography bytes:    %10d\n", $3)}' $MGDATA/STATS.wc
fi

$CAT <<END-OF-COMMENTS

Commands to the mgquery search engine begin with an initial period.
Anything else is interpreted as a keyword search string.

Useful mgquery commands:
	.quit				# terminate
	.help 				# help

	.set pager less			# single screen output paging
	.set pager cat			# continuous output to screen
	.set pager "cat >>file"		# continuous output to file

	.set query boolean		# Boolean instead of ranked searches

	.set mode hilite		# highlighted match strings
	.set hilite_style bold		# bold highlighting
	.set hilite_style underline	# underlined highlighting

HINT:	Commands that you always issue can be put into your ~/.mgrc file
	which mgquery will read on startup.

Each output entry is prefixed by a World Wide Web Universal Resource
Locator (URL) address that uniquely identifies the location of the
bibliography file.  Local users can see these files directly: just
change the prefix ftp://ftp.math.utah.edu to /u/ftp to create a local
file name.

With mg version 1.2x, "words" to be searched for may contain letters,
digits, hyphen (minus), underscore, backslash, or apostrophe.

To search for a BibNet-style BibTeX citation label, such as
Knuth:1992:LP, use Boolean search:

	.set query boolean
	knuth & 1992 & lp

========================================================================
END-OF-COMMENTS
}

### ====================================================================
### Provide a default definition of BIBSEARCHPATH, the directory where the mg
### database subdirectories reside.  However, BIBSEARCHPATH can be redefined
### by the user in the shell environment, or later by a command-line
### option.

setbibsearchpath

INITIALBIBSEARCHPATH=$BIBSEARCHPATH
BIBSEARCHPATH=

### ====================================================================
### Handle any command-line options:
quiet=0
while $TEST $# -gt 0
do
	debugreport parsing option: $1

	case $1 in
	--bibsearchpath | --bibsearchpat | --bibsearchpa | \
	--bibsearchp | --bibsearch | --bibsearc | --bibsear | --bibsea | \
	--bibse | --bibs | --bib | --bi | --b | -bibsearchpath | \
	-bibsearchpat | -bibsearchpa | -bibsearchp | -bibsearch | \
	-bibsearc | -bibsear | -bibsea | -bibse | -bibs | -bib | -bi | \
	-b)
		shift
		if $TEST -n "$BIBSEARCHPATH"
		then
			BIBSEARCHPATH=$BIBSEARCHPATH:$1
		else
			BIBSEARCHPATH=$1
		fi
		BIBSEARCHPATH="`expandpath $BIBSEARCHPATH`"
		debugreport BIBSEARCHPATH = $BIBSEARCHPATH
		;;

	--debug | --debu | --deb | --de | --d | -debug | -debu | -deb | -de | -d )
		DEBUG=yes
		;;

	--help | --hel | --he | --h | --'?' | -help | -hel | -he | -h | -'?' )
		help
		exit 0
		;;

	--quiet | --quie | --qui | --qu | --q | -quiet | -quie | -qui | -qu | -q)
		quiet=1
		;;

	--version | --versio | --versi | --vers | --ver | --ve | --v | -version | -versio | -versi | -vers | -ver | -ve | -v )
		echo This is $0 "version $VERSIONNUMBER $VERSIONDATE"
		exit 0
		;;

	-*)
		error "unrecognized option: [$1]"
		;;

	*)			# this had better be a database name
		break
		;;
	esac
	shift
done

$TEST $# -gt 1 && error expected only ONE database name


## Use the initial search path if no --bibsearchpath options were
## given.
$TEST -z "$BIBSEARCHPATH" && BIBSEARCHPATH="$INITIALBIBSEARCHPATH"

BIBSEARCHPATH="`expandpath $BIBSEARCHPATH`"

dirpathcheck $BIBSEARCHPATH

database=$1
if $TEST -z "$database"
then
	database="`$CAT $BIBSEARCHPATH/DEFAULT 2>/dev/null || echo tug"
	debugreport default database: $database
	BIBSEARCHPATH=$DEFAULTBIBSEARCHPATH
fi

MGDIR=`findinpath $BIBSEARCHPATH $database`
debugreport MGDIR = $MGDIR


MGDATA=$MGDIR/mgdata/bibfiles
export MGDATA
dircheck $MGDATA

###??? echo DEBUG: quiet = $quiet

if $TEST $quiet -ne 1
then
    ##??? echo DEBUG: welcome message should appear here
    welcome
fi

### [13-May-1997] cd to the data directory, in case there are
### files named database.* in the user's current directory
cd $MGDATA
debugreport working directory = MGDATA = $MGDATA

debugreport invoking: $MGQUERY bibfiles
$MGQUERY bibfiles
